#!/bin/bash
#SBATCH --job-name rechess  # name of the job
#SBATCH -e slurm-%j.err     # error file-path
#SBATCH -o slurm-%j.out     # output file-path
#SBATCH --nodes=1           # number of work nodes
#SBATCH --ntasks=3          # number of tasks, we need to specify --ntasks=1 for use of | and & in bash
#SBATCH --mem=128G          # reserved RAM
#SBATCH --cpus-per-task=4   # number of CPU cores per task
#SBATCH --gres=gpu:2        # number of GPUs
#SBATCH --time=2-02:00:00   # Time limit for the entire job (increased to 2d 2h)
#SBATCH --nodelist=workg02  # The 80 GB A100 GPUs are on node workg02

## NOTE: --cpus-per-task is logical cores rather than physical (Multithreading)!
## Add --cpus-per-task=<even-no> or ${SLURM_CPUS_PER_TASK} to each parallel task (preferred)
## Or disable multithreading: ##SBATCH --hint=nomultithread
## Switched to separate scripts and bash jobs for concurrent jobs
## This version is not to be run in parallel to another slurm batch job with the same script! (copyrepo.sh)

KUERZEL=$1
REPOPATH=$2

# Check argurment number
if [ "$#" -ne 2 ]; then
  echo "Found incorrect number of arguments ($#)."
  echo "Please specify:"
  echo "  - a user shorthand"
  echo "  - a re-CHESS repo zip file path (to copy from)"
  printf '\tcopyrepo.sh <user-shorthand> <repo-path>\n'
  exit 1
fi

# Check if the second argument is an existing file
if [ ! -f $2 ]; then
  echo "File $2 does not exist or is not a file."
  echo "Please specify a user shorthand and the re-CHESS repo zip file path:"
  printf '\tcopyrepo.sh <user-shorthand> <repo-path>\n'
  exit 1
fi

# Useful debug Information
srun --ntasks=1 echo "Tested: <running CHESS on SDS-BIRD>" # place a little reminder on what was tested here.
srun --ntasks=1 hostname
srun --ntasks=1 nvidia-smi
srun --ntasks=1 echo ${CUDA_VISIBLE_DEVICES}

# Create user directory on /zpool1/slurm_data/ if it doesn't exist yet
srun --ntasks=1 $(if [ -d /zpool1/slurm_data/${KUERZEL} ]; then rm -r /zpool1/slurm_data/${KUERZEL}/*; else mkdir /zpool1/slurm_data/${KUERZEL}; fi)

# Create directory for the job
srun --ntasks=1 mkdir /zpool1/slurm_data/${KUERZEL}/re-chess

RECHESSROOT=/zpool1/slurm_data/${KUERZEL}/re-chess

# Copy re-CHESS repo to the directory in /zpool1/slurm_data/ and unpack
sbcast $2 /zpool1/slurm_data/${KUERZEL}/re-chess.zip
srun --ntasks=1 unzip /zpool1/slurm_data/${KUERZEL}/re-chess.zip -d /zpool1/slurm_data/${KUERZEL}

# Download and unpack Ollama
srun --ntasks=1 ls -la /zpool1/slurm_data/${KUERZEL}
srun --ntasks=1 mkdir /zpool1/slurm_data/${KUERZEL}/ollama
srun --ntasks=1 mkdir /zpool1/slurm_data/${KUERZEL}/vllm
srun --ntasks=1 curl -f -s -S -L 'https://ollama.com/download/ollama-linux-amd64.tgz' -o "${RECHESSROOT}/ollama/ollama-linux-amd64.tgz"
srun --ntasks=1 tar -xzvf ${RECHESSROOT}/ollama/ollama-linux-amd64.tgz -C ${RECHESSROOT}/ollama
OLPATH=${RECHESSROOT}/ollama
OLSLOGFILE=/dev/null # log file for ollama serve (all occurences)
OLPLOGFILE=/dev/null # log file for ollama pull (all occurences)
export OLLAMA_MODELS=${OLPATH}/models
export OLLAMA_KEEP_ALIVE=-1

# Pull/copy llama3:70b, typically takes 4.5 to 5 mins
srun --ntasks=1 --time='00:07:00' ollamapull.sh ${OLPATH}
#srun --exclusive --time='00:06:00' --ntasks=1 --nodes=${SLURM_NNODES} --cpus-per-task=${SLURM_CPUS_PER_TASK} ${OLPATH}/./bin/ollama serve &> ${OLSLOGFILE} &
#TASKPID=$!
# sleep 10
#srun --ntasks=4 sleep 10 & # this seems incredibly stupid, but it ain't stupid if it work, right?
#srun --ntasks=1 --nodes=${SLURM_NNODES} --cpus-per-task=${SLURM_CPUS_PER_TASK} sleep 10
#wait %2
# srun --ntasks=1 sleep 10 ; echo 'llama3:70b' 'mxbai-embed-large' 'nomic-embed-text' | xargs -n1  ${OLPATH}/./bin/ollama pull &
#srun --ntasks=1 --nodes=${SLURM_NNODES} --cpus-per-task=${SLURM_CPUS_PER_TASK} ${OLPATH}/./bin/ollama pull 'llama3:70b' &
#srun --ntasks=1 --nodes=${SLURM_NNODES} --cpus-per-task=${SLURM_CPUS_PER_TASK} ${OLPATH}/./bin/ollama pull 'mxbai-embed-large' &
#srun --ntasks=1 --nodes=${SLURM_NNODES} --cpus-per-task=${SLURM_CPUS_PER_TASK} ${OLPATH}/./bin/ollama pull 'nomic-embed-text' &
#wait %2 %3 %4
#kill ${TASKPID}
# --dependency=after:1+1 ## resolution is 1 minute rounded up, job 1 was probably started at some point
# --time='00:06:00' ## download takes about 4.5 mins, increase to this with waiting
#srun ${OLPATH}/./bin/ollama serve &> ${OLSLOGFILE} & sleep 10 ; ${OLPATH}/./bin/ollama pull 'mxbai-embed-large' ; kill %1 ; echo "mxbai-embed-large pulled"
#srun ${OLPATH}/./bin/ollama serve &> ${OLSLOGFILE} & sleep 10 ; ${OLPATH}/./bin/ollama pull 'nomic-embed-text' ; kill %1 ; echo "nomic-embed-text pulled"
# sbcast $3 ${OLPATH}/models.zip
# srun unzip ${OLPATH}/models.zip -d ${OLPATH}

# Download the BIRD & Spider dataset
BIRDPATH=${RECHESSROOT}/CHESS/data/BIRD
SPIDERPATH=${RECHESSROOT}/CHESS/data/Spider
SPIDERTWOPATH=${RECHESSROOT}/CHESS/data/Spider2
srun --ntasks=1 ls -la /zpool1/slurm_data/${KUERZEL}
srun --ntasks=1 $(if [ ! -d ${BIRDPATH} ]; then mkdir ${BIRDPATH}; else echo "Directory ${BIRDPATH} exists."; fi)
srun --ntasks=1 $(if [ ! -d ${SPIDERPATH} ]; then mkdir ${SPIDERPATH}; else echo "Directory ${SPIDERPATH} exists."; fi)
# srun --ntasks=1 $(if [ ! -d ${SPIDERTWOPATH} ]; then mkdir ${SPIDERPATH}; else echo "Directory ${SPIDERPATH} exists."; fi)
srun --ntasks=1 curl -f -s -S -L 'https://bird-bench.oss-cn-beijing.aliyuncs.com/dev.zip' -o "${BIRDPATH}/dev.zip"
#srun --ntasks=1 curl -f -s -S -L 'https://bird-bench.oss-cn-beijing.aliyuncs.com/train.zip' -o "${BIRDPATH}/train.zip"
srun --ntasks=1 curl -f -s -S -L 'https://drive.usercontent.google.com/download?id=1403EGqzIDoHMdQF4c9Bkyl7dZLZ5Wt6J&confirm=t' -o "${SPIDERPATH}/spider_data.zip"
#srun --ntasks=1 curl -f -s -S -L 'https://drive.usercontent.google.com/download?id=1coEVsCZq-Xvj9p2TnhBFoFTsY-UoYGmG&confirm=t' -o '${SPIDERTWOPATH}/local_sqlite.zip' # for spider 2.0

# Unpack BIRD & Spider datasets
srun --ntasks=1 unzip ${BIRDPATH}/dev.zip -d ${BIRDPATH}
#srun --ntasks=1 unzip ${BIRDPATH}/train.zip -d ${BIRDPATH}
srun --ntasks=1 unzip ${SPIDERPATH}/spider_data.zip -d ${SPIDERPATH}

# Unpack BIRD specific sub-directory
srun --ntasks=1 mv ${BIRDPATH}/dev_20240627 ${BIRDPATH}/dev
#srun --ntasks=1 rm ${BIRDPATH}/dev_20240627
srun --ntasks=1 unzip ${BIRDPATH}/dev/dev_databases.zip -d ${BIRDPATH}/dev
#srun --ntasks=1 unzip ${BIRDPATH}/train/train_databases.zip -d ${BIRDPATH}/train
srun --ntasks=1 cp "${RECHESSROOT}/CHESS/data/dev/sub_sampled_bird_dev_set.json" "${BIRDPATH}/dev/sub_sampled_bird_dev_set.json"

# Unpack Spider 2.0 specific sub-directory
# TODO, dataset spread out over download above and github repo...

# Create a Python venv and install requirements
#srun --ntasks=1 py3versions --default
#srun --ntasks=1 py3versions --installed
PYTHONVER=$(py3versions --default)
srun --ntasks=1 ${PYTHONVER} -m venv ${RECHESSROOT}/CHESS/venv
VENVPATH=${RECHESSROOT}/CHESS/venv
OLDERPWD=${PWD}
source "${VENVPATH}/bin/activate" ;
# srun which pip3.11 ; # for debugging purposes
srun --ntasks=1 pip3.11 install -r "${RECHESSROOT}/CHESS/requirements.txt"

# Load nl2sql model via vllm
srun --time='00:05:00' --ntasks=1 ${PYTHONVER} -m vllm.entrypoints.openai.api_server --model "AI4DS/NL2SQL_DeepSeek_33B" --load-format safetensors --dtype bfloat16 --max-model-len 8192 --download-dir "${RECHESSROOT}/vllm"
srun --ntasks=1 ls -la "${VENVPATH}/lib/${PYTHONVER}/site-packages"
deactivate
# srun which pip3.11 ; # for debugging purposes
#srun cd ${RECHESSROOT}/CHESS ; source "${VENVPATH}/bin/activate" ; which pip3.11 ; deactivate ; cd ${OLDERPWD}
#srun cd ${RECHESSROOT}/CHESS ; source "${VENVPATH}/bin/activate" ; pip3.11 install -r "${RECHESSROOT}/CHESS/requirements.txt" ; deactivate ; cd ${OLDERPWD}


# copy the .env.rechess file to .env
srun --ntasks=1 cp ${RECHESSROOT}/CHESS/.env-rechess-bird-sds ${RECHESSROOT}/CHESS/.env
#srun cp ${RECHESSROOT}/CHESS/.env-rechess-spider ${RECHESSROOT}/CHESS/.env

# run preprocessing script
# srun cd ${RECHESSROOT}; source ${VENVPATH}/bin/activate; ${OLPATH}/./bin/ollama serve &> ${OLSLOGFILE} & sleep 10; /run/./run-preprocess.sh ; kill %1; deactivate; cd $OLDPWD
srun --ntasks=1 --time='2-00:00:00' runchess.sh ${RECHESSROOT} ${PYTHONVER}

#source ${VENVPATH}/bin/activate
#cd ${RECHESSROOT}/CHESS
#srun --time='10:00:00' --ntasks=1 --cpus-per-task=${SLURM_CPUS_PER_TASK} ${OLPATH}/./bin/ollama serve &> ${OLSLOGFILE} &
#srun --time='10:00:00' --ntasks=1 --cpus-per-task=${SLURM_CPUS_PER_TASK} ${PYTHONVER} -m vllm.entrypoints.openai.api_server --model "meta-llama/Meta-Llama-3-70B-Instruct" --dtype float32 --download-dir "${RECHESSROOT}/vllm" &
#srun --ntasks=1 --cpus-per-task=${SLURM_CPUS_PER_TASK} sleep 10 &
#wait %3
#srun --ntasks=1 --cpus-per-task=${SLURM_CPUS_PER_TASK} run/./run_preprocess.sh &
# run the agent script
#wait %3
#srun --time='00:02:00' --ntasks=1 --cpus-per-task=${SLURM_CPUS_PER_TASK} run/run_main_ir_ss_cg_bird-oss.sh &
#wait
#deactivate
#cd $OLDERPWD

# gather results, potentially breaks because of filenames starting with '-' ?
OLDERPWD=${PWD}
cd ${RECHESSROOT}/CHESS
srun --ntasks=1 zip -r ${OLDERPWD}/results-${SLURM_JOB_ID}.zip results/*
cd ${OLDERPWD}

##srun /zpool1/
##srun ls -la /lib
##srun ls -la /bin
##srun ls -la /zpool1/slurm_data/${KUERZEL}/${SLURM_JOBID}
##srun ls -la ${OLPATH}
##srun ls -la ${OLPATH}/model

# Remove all the data of the job to free memory on the node
srun --ntasks=1 rm -r /zpool1/slurm_data/${KUERZEL}/*
